var wtapi;
var authorized = false;
var poTable = null;
var poDialer = null;


var navigationView = null;
var chatView = null;
var subscriptionsView = null;
var queuesView = null;

$(document).ready(function() {
    $("#auth-form").on('submit', onAuthFormSubmit);

    poTable = $("#po-table");
    poDialer = $("#po-dialer");

    navigationView = $("#navigation-view");
    chatView = $("#chat-view");
    subscriptionsView = $("#subscriptions-view");
    queuesView = $("#queues-view");

    // Restore previously saved credentials
    if (typeof(Storage) !== "undefined" &&
        typeof(localStorage.extension) === "string" &&
        typeof(localStorage.password) === "string" &&
        typeof(localStorage.url) === "string") {
        $("#auth-extension-input").val(localStorage.extension);
        $("#auth-password-input").val(localStorage.password);
        $("#auth-url-input").val(localStorage.url);
    }
});


// ----------------------------------------
// Authentication functionality
// ----------------------------------------

function onAuthFormSubmit() {
    if ($("#auth-connect-button").hasClass("disabled")) {
        return;
    }

    $("#auth-connect-button").addClass("disabled");

    var extension = $("#auth-extension-input").val();
    var password = $("#auth-password-input").val();
    var url = $("#auth-url-input").val();

    // Save settings in local storage
    if (typeof(Storage)!=="undefined") {
        localStorage.extension = extension;
        localStorage.password = password;
        localStorage.url = url;
    }

    var re = new RegExp("^(http|https)://", "i");
    if(!re.test(url)) {
        url = 'https://' + url;
    }

    wtapi = new WTAPI(extension, password, url);
    wtapi.on("connected", onConnected);
    wtapi.on("disconnected", onDisconnected);

    // Attendant console demo
    wtapi.telephony.on("devices_changed", onDevicesChanged);
    wtapi.telephony.on("call_added", onCallAdded);
    wtapi.telephony.on("call_updated", onCallUpdated);
    wtapi.telephony.on("call_terminated", onCallTerminated);

    var calls = wtapi.telephony.getActiveCalls();
    var devices = wtapi.telephony.getRegisteredDevices();
    poTable.potable({calls: calls, devices: devices});
    poTable.on("answer", onCallAnswer);
    poTable.on("decline", onCallDecline);
    poTable.on("hangup", onCallHangup);
    poTable.on("hold", onCallHold);
    poTable.on("resume", onCallResume);
    poTable.on("attendant_transfer", onCallAttendantTransfer);
    poTable.on("blind_transfer", onCallBlindTransfer);
    poTable.on("start_record", onCallStartRecord);
    poTable.on("stop_record", onCallStopRecord);
    poDialer.dialer({activeDevices: devices});
    poDialer.on("dial", onDial);

    // Navigation between demos and personal presence component
    navigationView.navigation();
    navigationView.on("presence_changed", onChangePresence);

    // Subscriptions demo
    wtapi.presence.on("presence_changed", onPresenceChanged)
    wtapi.presence.on("personal_presence_changed", onPersonalPresenceChanged)
    subscriptionsView.subscriptions();
    subscriptionsView.on("subscribe", onSubscribe);
    subscriptionsView.on("unsubscribe", onUnsubscribe);

    // Queues demo
    wtapi.queues.on("queue_added", onQueueAdded);
    wtapi.queues.on("queue_removed", onQueueRemoved);
    wtapi.queues.on("member_added", onQueueMemberAdded);
    wtapi.queues.on("member_updated", onQueueMemberUpdated);
    wtapi.queues.on("member_removed", onQueueMemberRemoved);
    wtapi.queues.on("call_added", onQueueCallAdded);
    wtapi.queues.on("call_removed", onQueueCallRemoved);
    wtapi.queues.on("user_join", onQueueUserJoin);
    wtapi.queues.on("user_left", onQueueUserLeft);

    queuesView.queues();
    queuesView.on("subscribe", onQueuesSubscribe);
    queuesView.on("unsubscribe", onQueuesUnsubscribe);
    queuesView.on("member_add", onQueueMemberAdd);
    queuesView.on("member_pause", onQueueMemberPause);
    queuesView.on("member_resume", onQueueMemberResume);
    queuesView.on("member_remove", onQueueMemberRemove);

    // Chat demo
    wtapi.chat.on("message_received", onChatMessageReceived);
    chatView.chat();
    chatView.on("chat", onChatCreated);
    chatView.on("send", onChatMessageSend);

    // Connect to PBX
    wtapi.connect();
    return false;
}

function onAuthorizationSuccess() {
    $("#auth-viewport").hide();
    $("#po-viewport").show();
    $("#navigation-view").show();

    $("#navigation-po").on("click", onNavigationPoClicked);
    $("#navigation-chat").on("click", onNavigationChatClicked);
    $("#navigation-subs").on("click", onNavigationSubsClicked);
    $("#navigation-queues").on("click", onNavigationQueuesClicked);

    setInterval(onIterateOneSecond, 1000);
}

function onAuthorizationFailure(error) {
    wtapi.disconnect();
    wtapi = null;
    $("#auth-connect-button").removeClass("disabled");
    onError(error);
}

function onIterateOneSecond() {

    onQueueUpdateDuration();
}

// ----------------------------------------
// Connectivity functionality
// ----------------------------------------

function onConnected() {
    // Subscriptions functionality
    wtapi.presence.getSubscriptions(onSubscriptionsChanged);

    if (authorized) {
        onReconnected();
    }

    if (!authorized) {
        authorized = true;
        onAuthorizationSuccess();
    }
}

function onReconnected() {
    navigationView.navigation("reconnected");
    subscriptionsView.subscriptions("reconnected");
    chatView.chat("reconnected");
    queuesView.queues("reconnected");

    // TODO: Merge into one widget
    poTable.potable("onConnected");
    poDialer.dialer("enable");
}

function onNavigationPoClicked() {
    onNavigationChanged();
    $("#po-viewport").show();
    $("#navigation-po").parent().addClass("active");
}

function onNavigationChatClicked() {
    onNavigationChanged();
    $("#chat-view").show();
    $("#navigation-chat").parent().addClass("active");
}

function onNavigationSubsClicked() {
    onNavigationChanged();
    $("#subscriptions-view").show();
    $("#navigation-subs").parent().addClass("active");
}

function onNavigationQueuesClicked() {
    onNavigationChanged();
    $("#queues-view").show();
    $("#navigation-queues").parent().addClass("active");
}

function onNavigationChanged() {
    $("#po-viewport").hide();
    $("#navigation-po").parent().removeClass("active");
    $("#chat-view").hide();
    $("#navigation-chat").parent().removeClass("active");
    $("#subscriptions-view").hide();
    $("#navigation-subs").parent().removeClass("active");
    $("#queues-view").hide();
    $("#navigation-queues").parent().removeClass("active");
}

function onDisconnected(error) {
    if (!authorized) {
        onAuthorizationFailure(error);
    }

    if (authorized) {
        navigationView.navigation("disconnected");
        subscriptionsView.subscriptions("disconnected");
        chatView.chat("disconnected");
        queuesView.queues("disconnected");

        // TODO: Merge into one widget
        poTable.potable("onDisconnected");
        poDialer.dialer("disable");
    }
}

// ----------------------------------------
// Attendant Console Demo
// ----------------------------------------

function onDial(event, device, to) {
    poTable.potable("disable");
    poDialer.dialer("disable");
    wtapi.telephony.call(device, to, onError);
}

function onCallAnswer(event, call, device) {
    poTable.potable("disable");
    poDialer.dialer("disable");
    wtapi.telephony.answer(device, call, onError);
}

function onCallAttendantTransfer(event, call, callTo) {
    poTable.potable("disable");
    poDialer.dialer("disable");
    wtapi.telephony.bridge(call, callTo, onError);
}

function onCallBlindTransfer(event, call, to) {
    poTable.potable("disable");
    poDialer.dialer("disable");
    wtapi.telephony.forward(call, to, onError);
}

function onCallDecline(event, call) {
    poTable.potable("disable");
    poDialer.dialer("disable");
    wtapi.telephony.hangup(call, onError);
}

function onCallHangup(event, call) {
    poTable.potable("disable");
    poDialer.dialer("disable");
    wtapi.telephony.hangup(call, onError);
}

function onCallHold(event, call) {
    poTable.potable("disable");
    poDialer.dialer("disable");
    wtapi.telephony.hold(call, onError);
}

function onCallResume(event, call) {
    poTable.potable("disable");
    poDialer.dialer("disable");
    wtapi.telephony.resume(call, onError);
}

function onCallStartRecord(event, call) {
    poTable.potable("disable");
    poDialer.dialer("disable");
    wtapi.telephony.startRecord(call, onError);
}
function onCallStopRecord(event, call) {
    poTable.potable("disable");
    poDialer.dialer("disable");
    wtapi.telephony.stopRecord(call, onError);
}

function onError(error) {
    if (wtapi && wtapi.isConnected()) {
        poTable.potable("enable");
        poDialer.dialer("enable");
    }

    if (error === null) {
        return;
    }

    $('#error-modal-reason').text(error);
    $('#error-modal').modal("show");
}

function onDevicesChanged(devices) {
    $(poTable).potable("onDevicesChanged", devices);
    $(poDialer).dialer("onDevicesChanged", devices);
}

function onCallAdded(call) {
    $(poTable).potable("onCallAdded", call);
}

function onCallUpdated(call) {
    $(poTable).potable("onCallUpdated", call);
}

function onCallTerminated(call) {
    $(poTable).potable("onCallTerminated", call);
}


// ----------------------------------------
// Subscriptions Demo
// ----------------------------------------

/**
 * Fires when user change presence in navigation widget.
 */
function onChangePresence(event, presence) {
    wtapi.presence.changePersonalPresence(presence, onChangePresenceCallback);
}

function onChangePresenceCallback() {

}

function onPresenceChanged(user, presence) {
    $(subscriptionsView).subscriptions("changePresence", user, presence);
}

function onPersonalPresenceChanged(presence) {
    navigationView.navigation("changePersonalPresence", presence);
}

function onSubscriptionsChanged(users) {
    $(subscriptionsView).subscriptions("changeSubscriptions", users);
}

function onSubscribe(event, users) {
    wtapi.presence.subscribe(users, onSubscriptionsChanged);
}

function onUnsubscribe(event, users) {
    wtapi.presence.unsubscribe(users, onSubscriptionsChanged);
}

// ----------------------------------------
// Chat Demo
// ----------------------------------------

function onChatMessageReceived(user, message) {
    chatView.chat("showChat", user);
    chatView.chat("showIncomingMessage", user, message);

    // Send delivery confirmation
    // PBX will hold messages until they will not be delivered
    wtapi.chat.sendDeliveryConfirmation(user, message);
}

function onChatMessageSend(event, user, body) {
    var builder = new WTAPI.Message.Builder();
    builder.withAutoGeneratedId();
    builder.withBody(body);
    var message = builder.build();

    wtapi.chat.send(user, message);
    chatView.chat("showOutgoingMessage", user, message);
}

function onChatCreated(event, jid) {

	var s = jid.split("@");
    if (s.length <= 1) {
    	jid = jid+'@'+wtapi.server;
    }
    var user = wtapi.roster.getUser(jid);
    if (user === null) {
        user = wtapi.roster.createUser(jid);
    }

    chatView.chat("showChat", user);
}

// ----------------------------------------
// Queues Demo
// ----------------------------------------

function onQueueAdded(queue) {
    queuesView.queues('addQueue', queue);
}

function onQueueRemoved(queue) {
    queuesView.queues('removeQueue', queue);
}

function onQueueMemberAdded(queue, member) {
    queuesView.queues('addMember', queue, member);
    queuesView.queues('drawQueue', queue);
}

function onQueueMemberUpdated(queue, member) {
    queuesView.queues('updateMember', queue, member);
}

function onQueueMemberRemoved(queue, member) {
    queuesView.queues('removeMember', queue, member);
}

function onQueueCallAdded(queue, member, call) {
    queuesView.queues('updateMemberCalls', queue, member);
}

function onQueueCallRemoved(queue, member, call) {
    queuesView.queues('updateMemberCalls', queue, member);
}

function onQueueUserJoin(queue, user) {
    queuesView.queues('addUser', queue, user);
}

function onQueueUserLeft(queue, user) {
    console.log('onQueueUserLeft');
    queuesView.queues('removeUser', queue, user);
}

function onQueuesSubscribe() {
    queuesView.queues('disableActions');
    wtapi.queues.subscribe(onQueuesSubscribed);
}

function onQueuesSubscribed() {
    queuesView.queues('switchToSubscribedState');
    queuesView.queues('enableActions');
}

function onQueuesUnsubscribe() {
    queuesView.queues('disableActions');
    wtapi.queues.unsubscribe(onQueuesUnsubscribed);
}

function onQueuesUnsubscribed() {
    queuesView.queues('switchToUnsubscribedState');
    queuesView.queues('enableActions');
}

function onQueueMemberAdd(event, queue, extension) {
    if(!extension) return false;
    var callback = function(error) {
        if (error) {
            onError(error);
        }

        queuesView.queues('enable');
    };

    queuesView.queues('disable');
    wtapi.queues.add(queue, extension, callback);
}

function onQueueMemberPause(event, queue, member) {
    var callback = function(error) {
        if (error) {
            onError(error);
        }

        queuesView.queues('enable');
    };

    queuesView.queues('disable');
    wtapi.queues.pause(queue, member, callback);
}

function onQueueMemberResume(event, queue, member) {
    var callback = function(error) {
        if (error) {
            onError(error);
        }

        queuesView.queues('enable');
    };

    queuesView.queues('disable');
    wtapi.queues.resume(queue, member, callback);
}

function onQueueMemberRemove(event, queue, member) {
    var callback = function(error) {
        if (error) {
            onError(error);
        }

        queuesView.queues('enable');
    };

    queuesView.queues('disable');
    wtapi.queues.remove(queue, member, callback);
}

function onQueueUpdateDuration() {
    var queues = wtapi.queues.getQueues();
    for (var i=0; i < queues.length; i++) {
        var queue = queues[i];
        var members = queue.getMembers();
        var users = queue.getUsers();

        for (var c=0; c < members.length; c++) {
            queuesView.queues('updateMemberTime', queue, members[c]);
        }

        for (var c=0; c < users.length; c++) {
            queuesView.queues('updateUserDuration', queue, users[c]);
        }
    }
}

