$.widget("wildix.subscriptions", {
    _create: function () {
        this._presences = {};
        this._empty_presence = new WTAPI.Presence();
        this._columns = this.element.find("table td ul");
        this._inputEl = this.element.find(".subscriptions-input");
        this._inputEl.on("keypress", $.proxy(this._onInputKeyPress, this));
        this._addButton = this.element.find(".subscriptions-add-button");
        this._addButton.on("click", $.proxy(this._onAddButtonClick, this));
        this._removeButton = this.element.find(".subscriptions-remove-button");
        this._removeButton.on("click", $.proxy(this._onRemoveButtonClick, this));
        this._infoButton = this.element.find(".subscriptions-info-button");
        this._infoButton.on("click", $.proxy(this._onInfoButtonClick, this));

        $.template("subscriptionsItem", '<li data-extension="${extension}"><a href="#">' +
            '<i class="status offline"></i>' +
            '<span>${name} <i></i></span>' +
            '<i class="status ringing" style="display: none;"></i>' +
            '<i class="status talking" style="display: none;"></i>' +
            '<i class="status location" style="display: none;"></i>' +
        '</a></li>');
    },

    // --------------------
    // Public API
    // --------------------

    changePresence: function(user, presence) {
        // TODO: Remove a list of unused presences

        var extension = user.getExtension();
        var el = this.element.find("li[data-extension='"+extension+"']");

        this._setPresence(el, presence);
        this._presences[extension] = presence;
    },

    changeSubscriptions: function(users) {
        for (var i=0; i < this._columns.length; i++) {
            var column = this._columns[i];
            $(column).empty();
        }

        for (var i=0; i < users.length; i++) {
            var user = users[i];
            var extension = user.getExtension();
            var column = this._columns[i % this._columns.length];
            var el = $.tmpl("subscriptionsItem", {extension: extension, name: extension}).appendTo(column);
            var presence = this._presences[extension];
            this._setPresence(el, presence ? presence : this._empty_presence);
        }
    },

    disconnected: function() {
        this._inputEl.attr("disabled", true);
        this._addButton.addClass("disabled");
        this._removeButton.addClass("disabled");
    },

    reconnected: function() {
        this._inputEl.attr("disabled", false);
        this._addButton.removeClass("disabled");
        this._removeButton.removeClass("disabled");
    },

    // --------------------
    // Private functionality
    // --------------------

    _setPresence: function(node, presence) {
        var el = $(node);
        var anchorEl = el.find("a:first");
        var showText = "";
        var showEl = el.find("i:first");
        var statusEl = el.find("span i");
        var statusText = "";
        var ringingEl = el.find("i.ringing");
        var talkingEl = el.find("i.talking");
        var untilText = "";
        var locationEl = el.find("i.location");
        var locationText = "";
        var callText = "";

        // Set user presence status
        showEl.removeClass();
        showEl.addClass("status")

        if (presence.isOnline()) {
            if (presence.isAway()) {
                showEl.addClass("away");
                showText += "Away";
            } else if (presence.isDND()) {
                showEl.addClass("dnd");
                showText += "DND";
            } else {
                showEl.addClass("online");
                showText += "Online";
            }
        } else {
            if (presence.isAway()) {
                showEl.addClass("away-offline");
                showText += "Away - offline";
            } else if (presence.isDND()) {
                showEl.addClass("dnd-offline");
                showText += "DND - offline";
            } else if (presence.isRegisteredDevice()) {
                showEl.addClass("regd");
                showText += "Offline - reachable by phone";
            } else {
                showEl.addClass("offline");
                showText += "Offline";
            }
        }

        // Set status message
        if (presence.isStatusMessageAvailable()) {
            statusText += presence.getStatusMessage();
            statusEl.html(presence.getStatusMessage());
        } else {
            statusText += "-"
            statusEl.html("");
        }

        // Set user device status (ringing / talking)
        if (presence.isRinging()) {
            ringingEl.css("display", "inline-block");
        } else {
            ringingEl.css("display", "none");
        }
        if (presence.isTalking()) {
            talkingEl.css("display", "inline-block");
        } else {
            talkingEl.css("display", "none");
        }

        if (presence.isTalkingAndRinging()) {
            showText += " (ringing + talking)"
        } else if (presence.isRinging()) {
            showText += " (ringing)"
        } else if (presence.isTalking()) {
            showText += " (talking)"
        }

        // Set location
        if (presence.isLocationAvailable()) {
            var addr = presence.getLocation().getAddress();
            var lat = presence.getLocation().getLat();
            var lng = presence.getLocation().getLng();

            if (addr.length > 26) {
                addr = addr.substring(0, 26) + "...";
            }

            locationEl.css("display", "inline-block");
            locationText += addr
        } else {
            locationEl.css("display", "none");
            locationText = "-";
        }

        // Set call
        if (presence.isConnectedCallAvailable()) {
            var name = presence.getConnectedCall().getName();
            var number = presence.getConnectedCall().getNumber();
            var callText = '';

        	if(name && name != ''){
        		callText += name;
        	}
        	if(number && number !='' && callText != number){
        		if(callText != ''){
        			callText +=' ('+number+')';
            	}else{
            		callText += number;
            	}
        	}
        }

        // Set until
        if (presence.isStatusUntilAvailable()) {
            untilText += this._formatDate(presence.getStatusUntil());
        } else {
            untilText += "-"
        }

        if (anchorEl.hasClass("tooltip-applied")) {
            anchorEl.tooltip("destroy");
        } else {
            anchorEl.addClass("tooltip-applied");
        }

        // Set tooltip
        var description = "<div style='text-align:left;'>Status: "+ showText +"</div>" +
        	((callText != '') ? "<div style='text-align:left;'>Call: "+ callText +"</div>" : "") +
            "<div style='text-align:left;'>Message: "+ statusText +"</div>" +
            "<div style='text-align:left;'>Until: "+ untilText +"</div>" +
            "<div style='text-align:left;'>Location: "+ locationText +"</div>";
        anchorEl.tooltip({title: description, placement: "left", html: true});
    },

    _onInputKeyPress: function(event) {
        if (event.keyCode == 13) {
            this._onAddButtonClick();
            return false;
        }

        return true;
    },

    _onAddButtonClick: function() {
        var users = this._getInputUsers();

        this.element.trigger("subscribe", [users]);
        this._inputEl.val("");
    },

    _onRemoveButtonClick: function() {
        var users = this._getInputUsers();

        this.element.trigger("unsubscribe", [users]);
        this._inputEl.val("");
    },

    _onInfoButtonClick: function() {
        $("#subscriptions-help-modal").modal("show");
    },

    _getInputUsers: function() {
        var text = this._inputEl.val();
        return text.split(",");
    },

    _formatDate: function(date) {
        return date.getDate() + "/" + (date.getMonth() + 1) + "/" + date.getFullYear() + " " + date.getHours() + ":" + date.getMinutes();
    }
});