var wtapi;
var authorized = false;

$(document).ready(function() {
    window.statusEl = $('#status');
    window.queuesEl = $('#queues');

    var re = new RegExp("^(http|https)://", "i");
    if(!re.test(url)) {
        url = 'https://' + url;
    }

    wtapi = new WTAPI(extension, password, url);
    wtapi.on("connected", onConnected);
    wtapi.on("disconnected", onDisconnected);

    // Queues demo
    wtapi.queues.on("queue_added", onQueueAdded);
    wtapi.queues.on("queue_removed", onQueueRemoved);
    wtapi.queues.on("member_added", onQueueMemberAdded);
    wtapi.queues.on("member_updated", onQueueMemberUpdated);
    wtapi.queues.on("member_removed", onQueueMemberRemoved);
    wtapi.queues.on("call_added", onQueueCallAdded);
    wtapi.queues.on("call_removed", onQueueCallRemoved);
    wtapi.queues.on("user_join", onQueueUserJoin);
    wtapi.queues.on("user_left", onQueueUserLeft);
    wtapi.presence.on("presence_changed", onPresenceChanged);

    // Connect to PBX
    wtapi.connect();

    statusEl.css("min-height", $(window).height() - 90);
    queuesEl.css("min-height", $(window).height() - 50);

    statusEl.html("Connecting...");
    statusEl.show();
    queuesEl.hide();
    return false;
});


function onConnected() {
    if (authorized) {
        onReconnected();
    } else {
        authorized = true;
        onAuthorizationSuccess();
    }

    wtapi.roster.getRoster(function(){
        wtapi.queues.subscribe();
        wtapi.presence.subscribe([wtapi.getExtension()], function() {});
    });

    statusEl.hide();
    queuesEl.show();
}

function onReconnected() {
    statusEl.hide();
    queuesEl.show();
}

function onDisconnected(error) {
    if (!authorized) {
        onAuthorizationFailure(error);
    } else {
        onError(error);
    }
}

function onAuthorizationSuccess() {
    setInterval(onIterateOneSecond, 1000);
}

function onAuthorizationFailure(error) {
    wtapi.disconnect();
    wtapi = null;
    $("#auth-connect-button").removeClass("disabled");
    onError(error);
}

function onError(error) {
    statusEl.text(error);
    statusEl.show();
    queuesEl.hide();
}

function onIterateOneSecond() {
    onQueueUpdateDuration();
}

// ----------------------------------------
// Queues Demo
// ----------------------------------------

function onQueueAdded(queue) {
    var filter = window.queues || [];
    var exist = false;
    var sla_wait = 20;
    var sla_percentage = 90;
    for (var i=0; i < filter.length; i++) {
        if (filter[i]['id'] == parseInt(queue.getId())) {
            exist = true;
            sla_wait = filter[i]['sla_wait'];
            sla_percentage = filter[i]['sla_percentage'];
            break;
        }
    }

    if (exist) {
        var el = $("<div class=\"queue\" id=\"queue-" + queue.getId() +"\"></div>");
        el.appendTo($("#queues"));
        el.queue({queue: queue, sla_wait: sla_wait, sla_percentage: sla_percentage});
        el.on("add", jQuery.proxy(onAddMember, this, queue));
    }
}

function onQueueRemoved(queue) {
    $("#queue-" + queue.getId()).queue('destroy');
    $("#queue-" + queue.getId()).remove();
}

function onQueueMemberAdded(queue, member) {
    $("#queue-" + queue.getId()).queue('addMember', member);
    $("#queue-" + queue.getId()).queue('drawTable');
}

function onQueueMemberUpdated(queue, member) {
    $("#queue-" + queue.getId()).queue('updateMember', member);
}

function onQueueMemberRemoved(queue, member) {
    $("#queue-" + queue.getId()).queue('removeMember', member);
}

function onQueueCallAdded(queue, member, call) {
    $("#queue-" + queue.getId()).queue('addCall', member, call);
}

function onQueueCallRemoved(queue, member, call) {
    $("#queue-" + queue.getId()).queue('removeCall', member, call);
}

function onQueueUserJoin(queue, user) {
    $("#queue-" + queue.getId()).queue('addUser', user);
}

function onQueueUserLeft(queue, user) {
    $("#queue-" + queue.getId()).queue('removeUser', user);
}

function onQueueUpdateDuration() {
    if (window.queues) {
        for (var i=0; i < window.queues.length; i++) {
            $("#queue-" + window.queues[i]['id']).queue('time');
        }
    }
}

function onPresenceChanged(user, presence) {
    if (window.queues) {
        for (var i=0; i < window.queues.length; i++) {
            $("#queue-" + window.queues[i]['id']).queue('setPresence', user.getExtension(), presence);
        }
    }
}

function onAddMember(queue, event, extension) {
    var members = queue.getMembers();
    for (var i=0; i < members.length; i++) {
        if(members[i].getExtension() == extension) {
            wtapi.queues.remove(queue, members[i], function() {});
            return
        }
    }

    wtapi.queues.add(queue, extension, function() {});
}