/**
 * Settings window for queues demo.
 * Note:
 * There are four options for demo extension, password, url, queues which are takes
 * from global window context (window.extension, window.password ...), if such variables
 * not present in window, settings based on localStorage will used
 */

var settingsModal = null;

var options = {
    extension: '',
    password: '',
    url: '',
    queues: []
};

if (localStorage.getItem("queue-settings")){
    options = $.parseJSON(localStorage.getItem("queue-settings"));
}

if (window.extension == undefined || window.password == undefined || window.url == undefined){

    for (var option in options){
        if (options.hasOwnProperty(option)){
            window[option] = options[option];
        }
    }

    $(document).ready(function() {

        $(settingsHtml).appendTo($("#settings").show()).hide();
        settingsModal = $("#settings-modal");


        if (!options.extension || !options.password || !options.url){
            $('#status').hide();
            showSettings();
        }
        $('#settings-btn').on('click', function(){
            showSettings();
        });
        $('#settings-modal-save-button').on('click', onClickSettingsSave);
    });
}

function showSettings(){
    settingsModal.modal('show');

    $('#settings-modal-login').val(options.extension);
    $('#settings-modal-password').val(options.password);
    $('#settings-modal-host').val(options.url);
    for (var i = 0; i < 4; i++){
        if (options.queues[i]){
            $('#settings-modal-callgroup-'+(i+1)).val(options.queues[i].id);
            $('#settings-modal-sla-wait-'+(i+1)).val(options.queues[i].sla_wait);
            $('#settings-modal-sla-percentage-'+(i+1)).val(options.queues[i].sla_percentage);
        }
    }
}

function onClickSettingsSave(){
    options = {
        extension: $('#settings-modal-login').val(),
        password: $('#settings-modal-password').val(),
        url: $('#settings-modal-host').val(),
        queues: []
    };
    for (var i = 0; i < 4; i++){
        options.queues.push({
            id:$('#settings-modal-callgroup-'+(i+1)).val(),
            sla_wait:$('#settings-modal-sla-wait-'+(i+1)).val(),
            sla_percentage:$('#settings-modal-sla-percentage-'+(i+1)).val(),
        });
    }
    localStorage.setItem("queue-settings", JSON.stringify(options));

    if(wtapi){
        wtapi.disconnect();
    }

    settingsModal.modal('hide');

    document.location.reload();
}


var settingsHtml =
'<div id="settings-modal" class="modal fade" tabindex="-1" role="dialog" aria-hidden="true">\n\
     <div class="modal-dialog">\n\
         <div class="modal-content">\n\
             <div class="modal-header">\n\
                 <button type="button" class="close" data-dismiss="modal" aria-hidden="true">×</button>\n\
                 <h4 class="modal-title">Settings</h4>\n\
             </div>\n\
             <div class="modal-body">\n\
                 <table>\n\
                     <tr>\n\
                         <td width="120px">Host</td>\n\
                         <td><input id="settings-modal-host" class="form-control" type="text"></td>\n\
                     </tr>\n\
                     <tr>\n\
                         <td>Extension</td>\n\
                         <td><input id="settings-modal-login" class="form-control" type="text"></td>\n\
                     </tr>\n\
                     <tr>\n\
                         <td>Password</td>\n\
                         <td><input id="settings-modal-password" class="form-control" type="password"></td>\n\
                     </tr>\n\
                     <tr>\
                         <td colspan="2">\
                             <table class="settings-modal-queues">\
                                 <tr>\
                                     <td>Callgroup Id</td><td>SLA wait</td><td>SLA percentage</td>\
                                 </tr>\
                                 <tr>\
                                     <td><input id="settings-modal-callgroup-1" class="form-control" type="text" ></td>\
                                     <td><input id="settings-modal-sla-wait-1" class="form-control" type="text"></td>\
                                     <td><input id="settings-modal-sla-percentage-1" class="form-control" type="text"></td>\
                                 </tr>\
                                 <tr>\
                                     <td><input id="settings-modal-callgroup-2" class="form-control" type="text" ></td>\
                                     <td><input id="settings-modal-sla-wait-2" class="form-control" type="text"></td>\
                                     <td><input id="settings-modal-sla-percentage-2" class="form-control" type="text"></td>\
                                 </tr>\
                                 <tr>\
                                     <td><input id="settings-modal-callgroup-3" class="form-control" type="text" ></td>\
                                     <td><input id="settings-modal-sla-wait-3" class="form-control" type="text"></td>\
                                     <td><input id="settings-modal-sla-percentage-3" class="form-control" type="text"></td>\
                                 </tr>\
                                 <tr>\
                                     <td><input id="settings-modal-callgroup-4" class="form-control" type="text" ></td>\
                                     <td><input id="settings-modal-sla-wait-4" class="form-control" type="text"></td>\
                                     <td><input id="settings-modal-sla-percentage-4" class="form-control" type="text"></td>\
                                 </tr>\
                             </table>\n\
                         </td>\n\
                     </tr>\n\
                 </table>\n\
             </div>\n\
             <div class="modal-footer">\n\
                 <button id="settings-modal-save-button" type="button" class="btn btn-primary fa fa-check">Ok</button>\n\
             </div>\n\
         </div>\n\
     </div>\n\
 </div>\n\
';
